import logging
import os
from django.core.mail import EmailMultiAlternatives
from django.conf import settings
from django.template import Context, TemplateDoesNotExist
from django.contrib.sites.models import Site
from django.template.loader import render_to_string
from django.utils.translation import get_language, activate
from django.db import models
from django.core.exceptions import ImproperlyConfigured
from django.contrib.auth.models import User

class LanguageStoreNotAvailable(Exception):
    pass

def get_email_directories(dir):
    directory_tree = False
    for name in os.listdir(dir):
        if os.path.isdir(os.path.join(dir, name)):
            if directory_tree == False:
                directory_tree = {}
            directory_tree[name] = get_email_directories(os.path.join(dir, name))
    return directory_tree

    
def send_templated_email(recipients, template_path, context=None,
                    from_email=settings.DEFAULT_FROM_EMAIL,
                    fail_silently=False):
    """
        recipients can be either a list of emails or a list of users,
        if it is users the system will change to the language that the
        user has set as theyr mother toungue
    """
    current_language = get_language()
    current_site = Site.objects.get(id=settings.SITE_ID)

    default_context = context or {}
    default_context["current_site"] = current_site
    default_context["STATIC_URL"] = settings.STATIC_URL

    subject_path = "%s/short.txt" % template_path
    text_path = "%s/email.txt" % template_path
    html_path = "%s/email.html" % template_path

    for recipient in recipients:
        # if it is user, get the email and switch the language
        if isinstance(recipient, User):
            email = recipient.email
            try:
                language = get_users_language(recipient)
            except LanguageStoreNotAvailable:
                language = None

            if language is not None:
                # activate the user's language
                activate(language)
        else:
            email = recipient

        # populate per-recipient context
        context = Context(default_context)
        context['recipient'] = recipient
        context['email'] = email

        # load email text and subject
        subject = render_to_string(subject_path, context)
        subject = "".join(subject.splitlines()) # this must be a single line
        text = render_to_string(text_path, context)

        msg = EmailMultiAlternatives(subject, text, from_email, [email])

        # try to attach the html variant
        try:
            body = render_to_string(html_path, context)
            if getattr(settings, "TEMPLATEDEMAILS_USE_PYNLINER", False):
                import pynliner
                body = pynliner.fromString(body)
            msg.attach_alternative(body, "text/html")
        except TemplateDoesNotExist:
            logging.info("Email sent without HTML, since %s not found" % html_path)

        msg.send(fail_silently=fail_silently)

        # reset environment to original language
        if isinstance(recipient, User):
            activate(current_language)


def get_users_language(user):
    """
    Returns site-specific language for this user. Raises
    LanguageStoreNotAvailable if this site does not use translated
    notifications.
    """
    if getattr(settings, 'NOTIFICATION_LANGUAGE_MODULE', False):
        try:
            app_label, model_name = settings.NOTIFICATION_LANGUAGE_MODULE.split('.')
            model = models.get_model(app_label, model_name)
            language_model = model._default_manager.get(user__id__exact=user.id)
            if hasattr(language_model, 'language'):
                return language_model.language
        except (ImportError, ImproperlyConfigured, model.DoesNotExist):
            raise LanguageStoreNotAvailable
    raise LanguageStoreNotAvailable



from django.conf.urls.defaults import *
from .views import *

urlpatterns = patterns('',
    url(r'^$', index, name="templated_emails_index"),
    url(r'^view(?P<path>[\w.+-_/]+)$', view, name="templated_email_view"),
)



# -*- coding: utf-8 -*-
from django.utils import unittest
from django.test import TestCase
from .parse_util import parse_string_blocks, replace_blocks, replace_string_blocks


test_extraction_string =  """{% block test %}tr
 st

 sdf{% endblock %}"""

test_extraction_string2 = """{% block test123 %}tr s123t sdf{% endblock %}"""

test_extraction_string3 = """{% block test123 %}
            tr s123t sdf
{% endblock %}

{% block test %}test{% endblock %}
"""

test_extraction_string4 = """{% extends "emails/email.html" %}\n{% load gidsy_tags %}\n{% load profile_tags %}\n{% load account_tags %}\n{% load currency_tags %}\n{% block title %}\n  {% trans "All the best," %}\n{% endblock %}{% block message_title %}{% trans "Sorry! Not enough people booked" %} "{{ activity }}."{% endblock %}\n{% block email_notification %}{% trans "Your activity will not be taking place" %}{% endblock %}\n{% block email_title %}{% random_greeting %} {{ user.first_name }},{% endblock %}\n{% block email_subtitle %}\n  <h3>\n    {% blocktrans with slot=booking.slot min_participants=slot.get_min_participants start_date=slot.start_time|date:"l, F j" start_time=slot.start_time|date:"P" %}\n      We\'re sorry! Not enough people booked &quot;{{ activity }}&quot; on <strong>{{ start_date }}</strong> at <strong>{{ start_time }}</strong> for it to happen.\n    {% endblocktrans %}\n  </h3>\n{% endblock %}\n{% block email_message %}\n    <p>\n      {% blocktrans with confirmed_bookings=slot.booked_spots %}\n        To avoid disappointment in future, make sure to check before booking an activity how far it is from getting its minimum number of participants. \n      {% endblocktrans %}\n    </p>\n    {% if other_activities %}\n    <p>\n      You may also be interested in these other [link: activities] that already have enough people attending: (list 3 activities here)\n    </p>\n    {% endif %}\n{% endblock %}\n\n{% block signature %}\n  {% trans "All the best," %}\n{% endblock %}"""
test_replace_string = """<html>{% block test123 %}{% endblock %}</html>"""


class BlockExtraction(TestCase):
    def test_extraction(self):
        self.assertEquals("""tr
 st

 sdf""", parse_string_blocks(test_extraction_string, {})["test"])

        self.assertEquals("tr s123t sdf", parse_string_blocks(test_extraction_string2, {})["test123"])
        data = parse_string_blocks(test_extraction_string3, {})
        self.assertEquals("""
            tr s123t sdf
""",data["test123"])
        self.assertEquals("""test""",data["test"])

    def test_full_extraction(self):
        data = parse_string_blocks(test_extraction_string4, {})
        print data
        self.assertTrue(data.get("message_title", False))
        self.assertTrue(data.get("email_notification", False))
        self.assertTrue(data.get("email_title", False))
        self.assertTrue(data.get("email_subtitle", False))
        self.assertTrue(data.get("signature", False))
        self.assertTrue(data.get("title", False))
                
    def test_replace_blocks(self):
        self.assertEqual("<html>test</html>", replace_string_blocks(test_replace_string, {"test123": "test"}))


from django.contrib.auth.models import User
from django.http import HttpResponseRedirect, Http404, HttpResponse
from django.shortcuts import render_to_response, get_object_or_404
from django.template import RequestContext
from django.conf import settings
from django.template.loader import get_template
from django.template.base import Template
from .utils import get_email_directories
from .parse_util import recursive_block_replace


def index(request, template_name="templated_emails/index.html"):
    if not request.user.is_superuser:
        raise Http404
    else:
        directory_tree = get_email_directories(settings.EMAIL_TEMPLATES_DIRECTORY)
        return render_to_response(template_name, {
            "directory_tree": directory_tree,
        }, context_instance=RequestContext(request))


def view(request, path, template_name="templated_emails/view.html"):
    if not request.user.is_superuser:
        raise Http404
    else:
        # get extends node
        # get all block nodes
        # do this recursive until no more extends
        # then place html in blocks
        rendered_subject = ""
        rendered_html = ""
        rendered_text = ""

        try:
            template = get_template("emails%s/email.html"%path)
            rendered_html = recursive_block_replace(template, {})
        except:
            pass # there might be no html

        try:
            template = get_template("emails%s/email.txt"%path)
            rendered_text = recursive_block_replace(template, {})
        except:
            pass # there might be no text file

        try:
            template = get_template("emails%s/short.txt"%path)
            rendered_subject = recursive_block_replace(template, {})
        except:
            pass # there might be no html

        return render_to_response(template_name, {
            "rendered_subject": rendered_subject,
            "rendered_html": rendered_html,
            "rendered_text": rendered_text,
        }, context_instance=RequestContext(request))



#!/usr/bin/env python
from setuptools import setup
import os

long_description = open(os.path.join(os.path.abspath(os.path.dirname(__file__)), 'README.rst')).read()

setup(
    name = 'templated-emails',
    version = "0.6.5",
    url = 'https://github.com/philippWassibauer/templated-emails',
    author = "Philipp Wassibauer",
    author_email = "phil@gidsy.com",
    license = 'MIT License',
    packages = ['templated_emails'],
    package_data={
        'templated_emails': [
            'templates/templated_emails/*.html',
        ]
    },
    description = 'Like django-notifications, but just for sending plain emails. Written because it is ennoying to fork other apps just to make an email into an HTML email',
    long_description=long_description,
    classifiers = ['Development Status :: 4 - Beta',
                   'Environment :: Web Environment',
                   'Framework :: Django',
                   'Intended Audience :: Developers',
                   'License :: OSI Approved :: MIT License',
                   'Topic :: Internet :: WWW/HTTP :: Dynamic Content'],
    install_requires=[
        'pynliner',
        'cssutils',
    ],
)



from django.template.loader_tags import ExtendsNode
from django.conf import settings
import re

def recursive_block_replace(template, data=None, replace_static_url=True, replace_trans=True,
                            replace_with=True, replace_if=True):
    #check the extends
    # first try to replace with current data
    if isinstance(template.nodelist[0], ExtendsNode):
        # load more data using the current blocks
        data = parse_blocks(template, data)
        data = fill_blocks(template, data)
        return recursive_block_replace(template.nodelist[0].get_parent(""), data)
    else:
        final_string = replace_blocks(template, data)
        if replace_static_url:
            final_string = final_string.replace("{{ STATIC_URL }}", settings.STATIC_URL)
        if replace_trans:
            p = re.compile('(\{% blocktrans .* %\})', re.IGNORECASE)
            final_string = p.sub('', final_string)

            p = re.compile('(\{% blocktrans %\})', re.IGNORECASE)
            final_string = p.sub('', final_string)

            p = re.compile('(\{% endblocktrans %\})', re.IGNORECASE)
            final_string = p.sub('', final_string)

            final_string = re.sub(r"\{% trans (.+) %\}", lambda x: x.group(1)[1:-1], final_string)
        if replace_with:
            p = re.compile('(\{% with .+ %\})', re.MULTILINE|re.IGNORECASE)
            final_string = p.sub('', final_string)

            p = re.compile('(\{% endwith %\})', re.MULTILINE|re.IGNORECASE)
            final_string = p.sub('', final_string)

        if replace_if:
            p = re.compile('(\{% if .+ %\})', re.MULTILINE|re.IGNORECASE)
            final_string = p.sub(lambda x: "<div class='if'>%s</div><div class='if-body'>"%x.group(1), final_string)

            p = re.compile('(\{% endif %\})', re.MULTILINE|re.IGNORECASE)
            final_string = p.sub(lambda x: "</div><div class='end-if'>%s</div>"%x.group(1), final_string)

        return final_string


def replace_blocks(template, data):
    template_string = read_file(template.nodelist[0].source[0].name)
    return replace_string_blocks(template_string, data)


def replace_string_blocks(string, data):
    for key in data:
        regex_string = '\{% block '+key+' %\}([\s\S\n\r\w]*?)\{% endblock %\}'
        p = re.compile(regex_string, re.MULTILINE|re.IGNORECASE)
        string = p.sub(data[key], string)

    # replace the last blocks
    p2 = re.compile('(\{% block [a-zA-Z0-9_]+ %\})', re.MULTILINE|re.IGNORECASE)
    string = p2.sub('', string)

    p2 = re.compile('(\{% endblock %\})', re.MULTILINE|re.IGNORECASE)
    string = p2.sub('', string)

    return string


def fill_blocks(template, data):
    return data


def parse_blocks(template, data):
    data = parse_string_blocks(read_file(template.nodelist[0].source[0].name),
                               data)
    return data


def parse_string_blocks(string, data):
    # find all blocks
    regex = re.compile("\{% block ([a-zA-Z0-9_]+) %\}([\s\S\n\r\w]*?)\{% endblock %\}",
                       re.DOTALL|re.MULTILINE|re.IGNORECASE)
    m = regex.findall(string)
    for item in m:
        print item[0]
        data[item[0]] = item[1]
    return data


def read_file(path):
    file = open(path, "r")
    return file.read()
